@@ -1,17 +1,115 @@
 package HTML::CalendarMonth;
+BEGIN {
+  $HTML::CalendarMonth::VERSION = '1.24';
+}
 
 use strict;
-use vars qw($VERSION @ISA);
-
-$VERSION = '1.23';
-
+use warnings;
 use Carp;
 
+BEGIN { $HTML::CalendarMonth::VERSION = 1.24 }
+
 use HTML::ElementTable 1.18;
 use HTML::CalendarMonth::Locale;
 use HTML::CalendarMonth::DateTool;
 
-@ISA = qw(HTML::CalendarMonth::Accessor HTML::ElementTable);
+use base qw( Class::Accessor HTML::ElementTable );
+
+my %Objects;
+
+# default complex attributes
+my %Calmonth_Attrs = (
+  head_m      => 1,     # month heading mode
+  head_y      => 1,     # year heading mode
+  head_dow    => 1,     # DOW heading mode
+  head_week   => 0,     # weak of year
+  year_span   => 2,     # default col span of year
+
+  today       => undef, # DOM, if not now
+  week_begin  => 1,     # what DOW (1-7) is the 1st DOW?
+
+  historic    => 1,     # if able to choose, use ncal/cal
+                        # rather than Date::Calc, which
+                        # blindly extrapolates Gregorian
+
+  alias       => {},    # what gets displayed if not
+                        # the default item
+
+  month       => undef, # these will get initialized
+  year        => undef,
+
+  locale      => 'en_US',
+  full_days   => 0,
+  full_months => 1,
+
+  datetool    => undef,
+
+  enable_css   => 1,
+  semantic_css => 0,
+
+  # internal muckety muck
+  _cal      => undef,
+  _itoch    => {},
+  _ctoih    => {},
+  _caltool  => undef,
+  _weeknums => undef,
+  _today    => undef,
+
+  dow1st   => undef,
+  lastday  => undef,
+  loc      => undef,
+
+  # deprecated
+  row_offset => undef,
+  col_offset => undef,
+);
+
+__PACKAGE__->mk_accessors(keys %Calmonth_Attrs);
+
+# Class::Accessor overrides
+
+sub set {
+  my($self, $key) = splice(@_, 0, 2);
+  if (@_ == 1) {
+    $Objects{$self}{$key} = $_[0];
+  }
+  elsif (@_ > 1) {
+    $Objects{$self}{$key} = [@_];
+  }
+  else {
+    Carp::confess("wrong number of arguments received");
+  }
+}
+
+sub get {
+  my $self = shift;
+  if (@_ == 1) {
+    return $Objects{$self}{$_[0]};
+  }
+  elsif ( @_ > 1 ) {
+    return @{$Objects{$self}{@_}};
+  }
+  else {
+    Carp::confess("wrong number of arguments received.");
+  }
+}
+
+sub _is_calmonth_attr { shift; exists $Calmonth_Attrs{shift()} }
+
+sub _set_defaults {
+  my $self = shift;
+  foreach (keys %Calmonth_Attrs) {
+    $self->$_($Calmonth_Attrs{$_});
+  }
+  $self;
+}
+
+sub DESTROY { delete $Objects{shift()} }
+
+# last dow col, first week row
+
+use constant LDC => 6;
+use constant FWR => 2;
 
 # alias
 
@@ -77,47 +175,34 @@ sub _date {
   return($self->month, $self->year);
 }
 
-# locale accessors
-sub locales                 { shift->loc->locales           }
-sub locale_days             { shift->loc->days              }
-sub locale_daynum           { shift->loc->daynum(@_)        }
-sub locale_narrow_days      { shift->loc->narrow_days       }
-sub locale_months           { shift->loc->months            }
-sub locale_daynums          { shift->loc->daynums           }
-sub locale_narrow_months    { shift->loc->narrow_months     }
-sub locale_minmatch         { shift->loc->minmatch          }
-sub locale_monthnum         { shift->loc->monthnum(@_)      }
-sub locale_monthnums        { shift->loc->monthnums         }
-sub locale_minmatch_pattern { shift->loc->minmatch_pattern  }
-sub locale_first_dow        { shift->loc->first_day_of_week }
-
 # class factory access
 
-sub class_element_table { 'HTML::ElementTable' }
-sub class_datetool { __PACKAGE__ . '::DateTool' }
-sub class_locale   { __PACKAGE__ . '::Locale' }
+use constant CLASS_HET      => 'HTML::ElementTable';
+use constant CLASS_DATETOOL => 'HTML::CalendarMonth::DateTool';
+use constant CLASS_LOCALE   => 'HTML::CalendarMonth::Locale';
 
 sub _gencal {
   # generate internal calendar representation
   my $self = shift;
 
   # new calendar...clobber day-specific settings
-  my $itoc = $self->_itoc({});
+  my $itoc = $self->_itoch({});
+  my $ctoi = $self->_ctoih({});
 
   # figure out dow of 1st day of the month as well as last day of the
   # month (uses date calculator backends)
   $self->_anchor_month();
 
   # row count for weeks in grid
-  my ($wcnt) = 0;
+  my $wcnt = 0;
 
   my ($dowc) = $self->dow1st;
-  my $skips  = $self->_caltool->skips;
+  my $skips  = $self->_caltool->_skips;
 
   # for each day
   foreach (1 .. $self->lastday) {
     next if $skips->{$_};
-    my $r = $wcnt + 2;
+    my $r = $wcnt + FWR;
     my $c = $dowc;
     # this is a bootstrap until we know the number of rows in the month.
     $itoc->{$_} = [$r, $c];
@@ -127,18 +212,18 @@ sub _gencal {
 
   $self->{_week_rows} = $wcnt;
 
-  my $row_extent = $wcnt + 2;
-  my $col_extent = 6;
+  my $row_extent = $wcnt + FWR;
+  my $col_extent = LDC;
   $col_extent += 1 if $self->head_week;
 
-  $self->extent($row_extent, $col_extent);
+  $self->SUPER::extent($row_extent, $col_extent);
 
   # table can contain the days now, so replace our bootstrap coordinates
   # with references to the actual elements.
   foreach (keys %$itoc) {
     my $cellref = $self->cell(@{$itoc->{$_}});
-    $self->itoc($_, $cellref);
-    $self->ctoi($cellref, $_);
+    $self->_itoc($_, $cellref);
+    $self->_ctoi($cellref, $_);
   }
 
   # week num affects month/year spans
@@ -146,11 +231,11 @@ sub _gencal {
 
   # month/year headers
   my $cellref = $self->cell(0, 0);
-  $self->itoc($self->month, $cellref);
-  $self->ctoi($cellref, $self->month);
+  $self->_itoc($self->month, $cellref);
+  $self->_ctoi($cellref, $self->month);
   $cellref = $self->cell(0, $width - $self->year_span);
-  $self->itoc($self->year,  $cellref);
-  $self->ctoi($cellref, $self->year);
+  $self->_itoc($self->year,  $cellref);
+  $self->_ctoi($cellref, $self->year);
 
   $self->item($self->month)->replace_content($self->item_alias($self->month));
   $self->item($self->year)->replace_content($self->item_alias($self->year));
@@ -172,24 +257,24 @@ sub _gencal {
     }
   }
   else {
-    $self->row($self->first_row)->mask(1);
+    $self->row(0)->mask(1);
   }
 
   # DOW headers
   my $trans;
-  my $days = $self->locale_days;
+  my $days = $self->loc->days;
   foreach (0..$#$days) {
     # Transform for week_begin 1..7
     $trans = ($_ + $self->week_begin - 1) % 7;
     my $cellref = $self->cell(1, $_);
-    $self->itoc($days->[$trans], $cellref);
-    $self->ctoi($cellref, $days->[$trans]);
+    $self->_itoc($days->[$trans], $cellref);
+    $self->_ctoi($cellref, $days->[$trans]);
   }
   if ($self->head_dow) {
     grep($self->item($_)->replace_content($self->item_alias($_)), @$days);
   }
   else {
-    $self->row($self->first_row + 1)->mask(1);
+    $self->row(1)->mask(1);
   }
 
   # week number column
@@ -199,13 +284,13 @@ sub _gencal {
     # already aliased, of course).
     $self->_gen_week_nums();
     my $ws;
-    my $row_count = $self->first_week_row;
+    my $row_count = FWR;
     foreach ($self->_numeric_week_nums) {
       $ws = "w$_";
       $self->item_alias($ws, $_) unless $self->item_aliased($ws);
       my $cellref = $self->cell($row_count, $self->last_col);
-      $self->itoc($ws, $cellref);
-      $self->ctoi($cellref, $ws);
+      $self->_itoc($ws, $cellref);
+      $self->_ctoi($cellref, $ws);
       $self->item($ws)->replace_content($self->item_alias($ws));
       ++$row_count;
     }
@@ -213,8 +298,8 @@ sub _gencal {
 
   # fill in days of the month
   my $i;
-  foreach my $r ($self->first_week_row .. $self->last_row) {
-    foreach my $c ($self->first_col .. $self->last_week_col) {
+  foreach my $r (FWR .. $self->last_row) {
+    foreach my $c (0 .. LDC) {
       $self->cell($r,$c)->replace_content($self->item_alias($i))
         if ($i = $self->item_at($r,$c));
     }
@@ -307,7 +392,7 @@ sub _datetool {
   my $self = shift;
   my $ct;
   if (! ($ct = $self->_caltool)) {
-    $ct = $self->_caltool($self->class_datetool->new(
+    $ct = $self->_caltool(CLASS_DATETOOL->new(
       year     => $self->year,
       month    => $self->month,
       weeknum  => $self->head_week,
@@ -375,7 +460,7 @@ sub _gen_week_nums {
   my @wnums = $fweek > $lweek ? ($fweek, 1 .. $lweek) : ($fweek .. $lweek);
 
   # do we have days above our first Thursday?
-  if ($self->row_of($ft[0]) != $self->first_week_row) {
+  if ($self->row_of($ft[0]) != FWR) {
     unshift(@wnums, $wnums[0] -1);
   }
 
@@ -403,29 +488,28 @@ sub row_items {
   # given a list of items, return all items in rows shared by the
   # provided items.
   my $self = shift;
-  my($item,$row,$col,$i,@i,%i);
-  foreach $item (@_) {
-    $row = ($self->coords_of($item))[0];
-    foreach $col ($self->first_col .. $self->last_col) {
-      $i = $self->item_at($row,$col) || next;
-      ++$i{$i};
+  my %items;
+  foreach my $item (@_) {
+    my $row = ($self->coords_of($item))[0];
+    foreach my $col (0 .. $self->last_col) {
+      my $i = $self->item_at($row, $col) || next;
+      ++$items{$i};
     }
   }
-  @i = keys %i;
-  @i ? @i : $i[0];
+  keys %items > 1 ? keys %items : (keys %items)[0];
 }
 
 sub col_items {
   # return all item cells in the columns occupied by the provided list
   # of items.
   my $self = shift;
-  $self->_col_items($self->first_row,$self->last_row,@_);
+  $self->_col_items(0, $self->last_row, @_);
 }
 
 sub daycol_items {
   # same as col_items(), but excludes header cells.
   my $self = shift;
-  $self->_col_items($self->first_week_row,$self->last_row,@_);
+  $self->_col_items(FWR, $self->last_row, @_);
 }
 
 sub _col_items {
@@ -433,16 +517,16 @@ sub _col_items {
   # in the columns occupied by the provided items. Does not return
   # empty cells.
   my($self, $rfirst, $rlast) = splice(@_, 0, 3);
+  my %items;
   my($item, $row, $col, %i);
-  foreach $item (@_) {
-    $col = ($self->coords_of($item))[1];
-    foreach $row ($rfirst .. $rlast) {
+  foreach my $item (@_) {
+    my $col = ($self->coords_of($item))[1];
+    foreach my $row ($rfirst .. $rlast) {
       my $i = $self->item_at($row,$col) || next;
-      ++$i{$i};
+      ++$items{$i};
     }
   }
-  my @i = keys %i;
-  $#i ? @i : $i[0];
+  keys %items > 1 ? keys %items : (keys %items)[0];
 }
 
 sub daytime {
@@ -454,7 +538,7 @@ sub daytime {
 }
 
 sub week_nums {
-  # return list of all week numbers
+  # return list of all week number labels
   my @wnums = map("w$_", shift->_numeric_week_nums);
   wantarray ? @wnums : \@wnums;
 }
@@ -469,14 +553,14 @@ sub _numeric_week_nums {
 sub days {
   # return list of all days of the month (1..$c->lastday).
   my $self = shift;
-  my $skips = $self->_caltool->skips;
+  my $skips = $self->_caltool->_skips;
   my @days = grep { !$skips->{$_} } (1 .. $self->lastday);
   wantarray ? @days : \@days;
 }
 
 sub dayheaders {
   # return list of all day headers (Su..Sa).
-  shift->locale_days;
+  shift->loc->days;
 }
 
 sub headers {
@@ -493,33 +577,13 @@ sub items {
             : [$self->headers, $self->days];
 }
 
-use constant first_col => 0;
-
-sub first_week_col { shift->first_col }
-
 sub last_col {
   # what's the max col of the calendar?
   my $self = shift;
-  $self->head_week ? $self->last_week_col + 1 : $self->last_week_col;
+  $self->head_week ? LDC + 1 : LDC;
 }
 
-sub last_week_col {
-  # what column does the last DOW fall in? Should be the same as
-  # last_col unless head_week is activated
-  shift->first_col + 6;
-}
-
-use constant first_row => 0;
-
-sub first_week_row {
-  # returns the first row containing days of the month. This used to
-  # take into account whether the header rows were active or not,
-  # but since masking was implemented this should always be offset 2
-  # from the first row (thereby taking into account the month/year
-  # and DOW rows).
-  my $w = 2;
-  shift->first_row + $w;
-}
+sub last_day_col { LDC }
 
 sub last_row {
   # last row of the calendar
@@ -527,7 +591,33 @@ sub last_row {
   return ($self->coords_of($self->lastday))[0];
 }
 
-sub last_week_row { last_row(@_) }
+*last_week_row = \&last_row;
+
+sub first_week_row { FWR };
+
+sub past_days {
+  my $self  = shift;
+  my $today = $self->_today;
+  if ($today < 0) {
+    return $self->days;
+  }
+  elsif ($today == 0) {
+    return;
+  }
+  return(1 .. $today);
+}
+
+sub future_days {
+  my $self  = shift;
+  my $today = $self->_today;
+  if ($today < 0) {
+    return;
+  }
+  elsif ($today == 0) {
+    return $self->days;
+  }
+  return($today .. $self->last_day);
+}
 
 # custom glob interfaces
 
@@ -551,7 +641,7 @@ sub item_day_row {
   my(%rows, @coords);
   for my $r (map { $self->row_of($_) } @_) {
     next if ++$rows{$r} > 1;
-    for my $c ($self->first_col .. $self->last_week_col) {
+    for my $c (0 .. 6) {
       push(@coords, ($r, $c));
     }
   }
@@ -567,13 +657,13 @@ sub item_week_nums {
 sub item_col {
   # return a glob of the cols of a list of items, including empty cells.
   my $self = shift;
-  $self->_item_col($self->first_row, $self->last_row, @_);
+  $self->_item_col(0, $self->last_row, @_);
 }
 
 sub item_daycol {
   # same as item_col(), but excludes header cells.
   my $self = shift;
-  $self->_item_col($self->first_week_row, $self->last_row, @_);
+  $self->_item_col(2, $self->last_row, @_);
 }
 
 sub _item_col {
@@ -602,15 +692,13 @@ sub item_box {
 sub all {
   # return a glob of all calendar cells, including empty cells.
   my $self = shift;
-  $self->box( $self->first_row => $self->first_col,
-              $self->last_row  => $self->last_col   );
+  $self->box( 0,0 => $self->last_row, $self->last_col );
 }
 
 sub alldays {
   # return a glob of all cells other than header cells
   my $self = shift;
-  $self->box( $self->first_week_row => $self->first_col,
-              $self->last_row       => $self->last_week_col );
+  $self->box( 2, 0 => $self->last_row, 6 );
 }
 
 sub allheaders {
@@ -625,7 +713,7 @@ sub coords_of {
   # convert an item into grid coordinates
   my $self = shift;
   croak "undefined value passed to coords_of()" if @_ && ! defined $_[0];
-  my $ref = $self->itoc(@_);
+  my $ref = $self->_itoc(@_);
   my @pos = ref $ref ? $ref->position : ();
   @pos ? (@pos[$#pos - 1, $#pos]) : ();
 }
@@ -633,14 +721,14 @@ sub coords_of {
 sub item_at {
   # convert grid coords into item
   my $self = shift;
-  $self->ctoi($self->cell(@_));
+  $self->_ctoi($self->cell(@_));
 }
 
-sub itoc {
+sub _itoc {
   # item to grid
   my($self, $item, $ref) = splice(@_, 0, 3);
   defined $item or croak "item required";
-  my $itoc = $self->_itoc;
+  my $itoc = $self->_itoch;
   if ($ref) {
     croak "Reference required" unless ref $ref;
     $itoc->{$item} = $ref;
@@ -648,11 +736,11 @@ sub itoc {
   $itoc->{$item};
 }
 
-sub ctoi {
+sub _ctoi {
   # cell reference to item
   my($self, $refstring, $item) = splice(@_, 0, 3);
   defined $refstring or croak "cell id required";
-  my $ctoi = $self->_ctoi;
+  my $ctoi = $self->_ctoih;
   if (defined $item) {
     $ctoi->{$refstring} = $item;
   }
@@ -670,83 +758,72 @@ sub col_of {
 }
 
 sub monthname {
-  # check/return month...returns name. Accepts 1-12, or Jan..Dec
+  # check/return month...returns name. Accepts month number or string.
   my $self = shift;
   return $self->month unless @_;
-  my(@mn, $month);
-  my $months   = $self->locale_months;
-  my $monthnum = $self->locale_monthnums;
-  my $minmatch = $self->locale_minmatch;
-  my $mmpat    = $self->locale_minmatch_pattern;
-  
-  foreach $month (@_) {
-    if ($month =~ /^\d+$/) {
-      $month >= 1 && $month <= 12 || return 0;  
-      push(@mn, $months->[$month-1]);
-    }
-    else {
-      if (exists $monthnum->{$month}) {
-        push(@mn, $month);
-      }
-      else {
-        # make one last attempt
-        if ($month =~ /^($mmpat)/) {
-          push(@mn, $minmatch->{$1});
-        }
-        else {
-          return undef;
-        }
-      }
-    }
+  my $loc = $self->loc;
+  my @names;
+  for my $m (@_) {
+    $m = ($m - 1) % 12 if $m && $m =~ /^\d+$/;
+    $m = $loc->monthname($m) || croak "month not found " . join(', ', @_);
+    return $m if @_ == 1;
+    push(@names, $m);
   }
-  $#mn > 0 ? @mn : $mn[0];
+  @names;
 }
 
 sub monthnum {
-  # check/return month, returns number. Accepts 1-12, or Jan..Dec
-  my $self = shift;
-  my $monthnum = $self->locale_monthnums;
-  my @mn;
-  push(@mn, map(exists $monthnum->{$_} ?
-                $monthnum->{$_}+1 : undef, $self->monthname(@_)));
-  $#mn > 0 ? @mn : $mn[0];
+  # check/return month, returns number. Accepts month number or string.
+  my $self   = shift;
+  my @months = @_ ? @_ : $self->month;
+  my $loc = $self->loc;
+  my @nums;
+  for my $m (@months) {
+    $m = ($m - 1) % 12 if $m && $m =~ /^\d+$/;
+    $m = $loc->monthnum($m);
+    croak "month not found ", join(', ', @_) unless defined $m;
+    $m += 1;
+    return $m if @_ == 1;
+    push(@nums, $m);
+  }
+  @nums;
 }
 
 sub dayname {
   # check/return day...returns name. Accepts 1..7, or Su..Sa
   my $self = shift;
-  @_ || croak "Day must be provided";
-  my(@dn, $day);
-  my $days = $self->locale_days;
-  my $daynum = $self->locale_daynums;
-  foreach $day (@_) {
-    if ($day =~ /^\d+$/) {
-      $day >= 1 && $day <= 7 || return undef;
-      # week_begin is at least 1, so skew is automatic
-      push(@dn, $days->[($day - 1 + $self->week_begin - 1) % 8]);
-    }
-    else {
-      $day = ucfirst(lc($day));
-      if (exists $daynum->{$day}) {
-        push(@dn, $day);
-      }
-      else {
-        return undef;
-      }
+  @_ || croak "day string or num required";
+  my $loc = $self->loc;
+  my @names;
+  for my $d (@_) {
+    if ($d =~ /^\d+$/) {
+      $d = (($d - 1) % 7) + $self->week_begin - 1;
     }
+    $d = $loc->dayname($d) || croak "day not found ", join(', ', @_);
+    return $d if @_ == 1;
+    push(@names, $d);
   }
-  $#dn > 0 ? @dn : $dn[0];
+  @names;
 }
 
 sub daynum {
   # check/return day number 1..7, returns number. Accepts 1..7,
   # or Su..Sa
   my $self = shift;
-  my $daynum = $self->locale_daynums;
-  my @dn;
-  push(@dn, map(exists $daynum->{$_} ?
-                $daynum->{$_}+1 : undef,$self->dayname(@_)));
-  $#dn > 0 ? @dn : $dn[0];
+  @_ || croak "day string or num required";
+  my $loc  = $self->loc;
+  my @nums;
+  for my $d (@_) {
+    if ($d =~ /^\d+$/) {
+      $d = (($d - 1) % 7) + $self->week_begin - 1;
+    }
+    $d = $loc->daynum($d);
+    croak "day not found ", join(', ', @_) unless defined $d;
+    $d += 1;
+    return $d if @_ == 1;
+    push(@nums, $d);
+  }
+  @nums;
 }
 
 # tests-n-checks
@@ -755,7 +832,7 @@ sub _dayheadcheck {
   # test day head names
   my($self, $name) = splice(@_, 0, 2);
   $name or croak "name missing";
-  return undef if $name =~ /^\d+$/;
+  return if $name =~ /^\d+$/;
   $self->daynum($name);
 }
 
@@ -780,7 +857,7 @@ sub new {
   my %parms = @_;
   my(%attrs, %tattrs);
   foreach (keys %parms) {
-    if (__PACKAGE__->is_calmonth_attr($_)) {
+    if (__PACKAGE__->_is_calmonth_attr($_)) {
       $attrs{$_} = $parms{$_};
     }
     else {
@@ -788,11 +865,11 @@ sub new {
     }
   }
 
-  my $self = __PACKAGE__->class_element_table->new(%tattrs);
+  my $self = CLASS_HET->new(%tattrs);
   bless $self, $class;
 
   # set defaults
-  $self->set_defaults;
+  $self->_set_defaults;
 
   my $month = delete $attrs{month};
   my $year  = delete $attrs{year};
@@ -810,14 +887,14 @@ sub new {
     $self->$k($attrs{$k}) if defined $attrs{$k};
   }
 
-  my $loc = $self->class_locale->new(
+  my $loc = CLASS_LOCALE->new(
     id          => $self->locale,
     full_days   => $self->full_days,
     full_months => $self->full_months,
   ) or croak "Problem creating locale " . $self->locale . "\n";
   $self->loc($loc);
 
-  my $dt = $self->class_datetool->new(
+  my $dt = CLASS_DATETOOL->new(
       year     => $self->year,
       month    => $self->month,
       weeknum  => $self->head_week,
@@ -826,24 +903,25 @@ sub new {
   );
   $self->_caltool($dt);
 
-  $self->week_begin($loc->first_day_of_week)
+  $self->week_begin($loc->first_day_of_week + 1)
     unless defined $attrs{week_begin};
 
-  my $dom_now = defined $attrs{today} ? $dt->dom_now(delete $attrs{today})
-                                      : $dt->dom_now;
-  $self->today($dom_now);
+  my $dom_now = defined $attrs{today} ? $dt->_dom_now(delete $attrs{today})
+                                      : $dt->_dom_now;
+  $self->_today($dom_now);
+  $self->today($dom_now) if $dom_now > 0;
 
   my $alias = $attrs{alias} || {};
   if ($self->full_days < 0) {
-    my @full   = $self->locale_days;
-    my @narrow = $self->locale_narrow_days;
+    my @full   = $self->loc->days;
+    my @narrow = $self->loc->narrow_days;
     for my $i (0 .. $#narrow) {
       $alias->{$full[$i]} = $narrow[$i];
     }
   }
   if ($self->full_months < 0) {
-    my @full   = $self->locale_months;
-    my @narrow = $self->locale_narrow_months;
+    my @full   = $self->loc->months;
+    my @narrow = $self->loc->narrow_months;
     for my $i (0 .. $#narrow) {
       $alias->{$full[$i]} = $narrow[$i];
     }
@@ -857,217 +935,112 @@ sub new {
   $self;
 }
 
+### overrides (our table is static)
+
+sub extent { }
+sub maxrow { shift->SUPER::maxrow }
+sub maxcol { shift->SUPER::maxcol }
+
 ### deprecated
 
-use constant row_offset => 0;
-use constant col_offset => 0;
+use constant row_offset     => 0;
+use constant col_offset     => 0;
+use constant first_col      => 0;
+use constant first_row      => 0;
+use constant first_week_col => 0;
+use constant last_week_col  => 6;
 
 ###
 
-{
-
-package HTML::CalendarMonth::Accessor;
-
-use strict;
-use vars qw($VERSION @ISA);
-
-$VERSION = '0.01';
-
-use Carp;
-
-use Class::Accessor;
-
-@ISA = qw(Class::Accessor);
-
-my %Objects;
-
-# default complex attributes
-my %Calmonth_Attrs = (
-  head_m      => 1,     # month heading mode
-  head_y      => 1,     # year heading mode
-  head_dow    => 1,     # DOW heading mode
-  head_week   => 0,     # weak of year
-  year_span   => 2,     # default col span of year
-
-  today       => undef, # DOM, if not now
-  week_begin  => 1,     # what DOW (1-7) is the 1st DOW?
-
-  historic    => 1,     # if able to choose, use ncal/cal
-                        # rather than Date::Calc, which
-                        # blindly extrapolates Gregorian
-
-  alias       => {},    # what gets displayed if not
-                        # the default item
-
-  month       => undef, # these will get initialized
-  year        => undef,
-
-  locale      => 'en_US',
-  full_days   => 0,
-  full_months => 1,
-
-  datetool    => undef,
-
-  enable_css   => 1,
-  semantic_css => 0,
-
-  # internal muckety muck
-  _cal      => undef,
-  _itoc     => {},
-  _ctoi     => {},
-  _caltool  => undef,
-  _weeknums => undef,
-
-  dow1st   => undef,
-  lastday  => undef,
-  loc      => undef,
-
-  # deprecated
-  row_offset => undef,
-  col_offset => undef,
-);
-
-__PACKAGE__->mk_accessors(keys %Calmonth_Attrs);
-
-# Class::Accessor overrides
-
-sub new {
-  my $class = shift;
-  my $self = $class->SUPER::new(@_);
-  foreach (sort keys %Calmonth_Attrs) {
-    $self->$_($Calmonth_Attrs{$_});
-  }
-  $self;
-}
-
-sub set {
-  my($self, $key) = splice(@_, 0, 2);
-  if (@_ == 1) {
-    $Objects{$self}{$key} = $_[0];
-  }
-  elsif (@_ > 1) {
-    $Objects{$self}{$key} = [@_];
-  }
-  else {
-    Carp::confess("wrong number of arguments received");
-  }
-}
-
-sub get {
-  my $self = shift;
-  if (@_ == 1) {
-    return $Objects{$self}{$_[0]};
-  }
-  elsif ( @_ > 1 ) {
-    return @{$Objects{$self}{@_}};
-  }
-  else {
-    Carp::confess("wrong number of arguments received.");
-  }
-}
-
-sub is_calmonth_attr { shift; exists $Calmonth_Attrs{shift()} }
-
-sub set_defaults {
-  my $self = shift;
-  foreach (keys %Calmonth_Attrs) {
-    $self->$_($Calmonth_Attrs{$_});
-  }
-  $self;
-}
-
-} # end HTML::CalendarMonth::Accessor
-
 1;
 
 __END__
 
 =head1 NAME
 
-HTML::CalendarMonth - Perl extension for generating and manipulating HTML calendar months
+HTML::CalendarMonth - Generate and manipulate HTML calendar months
 
 =head1 SYNOPSIS
 
  use HTML::CalendarMonth;
- use HTML::AsSubs;
-
- # Using HTML::AsSubs
- $c = HTML::CalendarMonth->new( month => 3, year => 69 );
- $c->item($c->year, $c->month)->attr(bgcolor => 'wheat');
- $c->item($c->year, $c->month)->wrap_content(font({size => '+2'}));
- $c->item(12, 16, 28)->wrap_content(strong());
- print $c->as_HTML;
 
  # Using regular HTML::Element creation
- $c2 = HTML::CalendarMonth->new( month => 8, year => 79 );
- $c2->item($c2->year, $c2->month)->attr(bgcolor => 'wheat');
- $f = HTML::Element->new('font', size => '+2');
- $c2->item($c2->year, $c2->month)->wrap_content($f);
- $c2->item_daycol('Su', 'Sa')->attr(bgcolor => 'cyan');
- print $c2->as_HTML;
+ my $c = HTML::CalendarMonth->new( month => 8, year => 2010 );
+ print $c->as_HTML;
 
  # Full locale support via DateTime::Locale
- $c3 HTML::CalendarMonth->new( month => 8, year => 79, locale => 'fr' );
- print $c3->as_HTML
+ my $c2 = HTML::CalendarMonth->new(
+   month  => 8,
+   year   => 2010,
+   locale => 'zu_ZA'
+ );
+ print $c2->as_HTML;
+
+ # HTML-Tree integration
+ my $tree = HTML::TreeBuilder->parse_file('cal.html');
+ $tree->find_by_attribute(class => 'hcm-calendar')->replace_with($c);
+ print $tree->as_HTML;
+
+ # clean up if you're not done, HTML::Element structures must be
+ # manually destroyed
+ $c->delete; $c2->delete;
 
 =head1 DESCRIPTION
 
 HTML::CalendarMonth is a subclass of HTML::ElementTable. See
 L<HTML::ElementTable(3)> for how that class works, for it affects this
-module on many levels. Like HTML::ElementTable, HTML::CalendarMonth
-behaves as if it were an HTML::ElementSuper, which is a regular
-HTML::Element with methods added to easily manipulate the appearance of
-the HTML table containing the calendar.
+module on many levels. Like HTML::ElementTable, HTML::CalendarMonth is
+an enhanced HTML::Element with methods added to facilitate the
+manipulation of the calendar table elements as a whole.
 
-The primary interaction with HTML::CalendarMonth is through I<items>. An
-I<item> is merely a symbol that represents the content of the cell of
-interest within the calendar. For instance, the element representing the
-14th day of the month would be returned by C<$c-E<gt>item(14)>.
-Similarly, the element representing the header for Monday would be
-returned by C<$c-E<gt>item('Mo')>. If the year happened to by 1984, then
-C<$c-E<gt>item(1984)> would return the cell representing the year. Since
+The primary interaction with HTML::CalendarMonth is through I<items>
+rather than cell coordinates like HTML::ElementTable uses. An I<item> is
+merely a string that represents the content of the cell of interest
+within the calendar. For instance, the element representing the 14th day
+of the month would be returned by C<$c-E<gt>item(14)>. Similarly, the
+element representing the header for Monday would be returned by C<$c-
+E<gt>item('Mo')>. If the year happened to by 2010, then C<$c-
+E<gt>item(2010)> would return the cell representing the year. Since
 years and particular months change frequently, it is probably more
 useful to take advantage of the C<month()> and C<year()> methods, which
-return the respective item symbol for the current calendar. In the prior
-example, using 1984, the following is equivalent: C<$c-E<gt>item($c-
-E<gt>year())>.
+return their respective values. The following is therefore the same as
+explicitely referencing the year: C<$c-E<gt>item($c- E<gt>year())>.
 
 Multiple cells of the calendar can be manipulated as if they were a
-single element. For instance, C<$c-E<gt>item(15)-E<gt>attr(bgcolor
-=E<gt> 'cyan')> would alter the background color of the cell
-representing the 15th. By the same token, C<$c-E<gt>item(15, 16, 17,
-23)-E<gt>attr(bgcolor =E<gt> 'cyan')> would do the same thing for all
-cells containing the item symbols passed to the C<item()> method.
+single element. For instance, C<$c-E<gt>item(15)-E<gt>attr(class =E<gt>
+'fancyday')> would alter the class of the cell representing the 15th. By
+the same token, C<$c-E<gt>item(15, 16, 17,
+23)-E<gt>attr(class =E<gt> 'fancyday')> would do the same thing for all
+cells containing the days passed to the C<item()> method.
 
-The calendar structure is still nothing more than a table structure; the
-same table structure provided by the HTML::ElementTable class. In
-addition to the I<item> based access methods above, calendar cells can
-still be accessed using row and column grid coordinates using the
-C<cell()> method provided by the table class. All coordinate-based
-methods in the table class are accessible to the calendar class.
+Underneath, the calendar is still nothing more than a table structure,
+the same as provided by the HTML::ElementTable class. In addition to the
+I<item> based access methods above, calendar cells can still be accessed
+using row and column grid coordinates using the C<cell()> method
+provided by the table class. All coordinate-based methods in the table
+class are accessible to the calendar class.
 
 The module includes support for week-of-the-year numbering, arbitrary
-1st day of the week definitions, and aliasing so that you can express
-any element in any language HTML can handle.
+1st day of the week definitions, and locale support.
 
 Dates that are beyond the range of the built-in time functions of perl
-are handled either by the ncal/cal command, Date::Calc, or Date::Manip.
-The presence of any one of these utilities and modules will suffice for
-these far flung date calculations. If you want to use week-of-year
-numbering, then ncal or either one of the date modules is required.
+are handled either by the ncal/cal command, Date::Calc, DateTime, or
+Date::Manip. The presence of any one of these utilities and modules will
+suffice for these far flung date calculations. One of these utilities
+(with the exception of 'cal') is also required if you want to use week-of-
+year numbering.
 
 Full locale support is offered via DateTime::Locale. For a full list of
-supported locale id's, look at HTML::CalendarMonth::Locale->locales() or
-DateTime::Locale->ids().
+supported locale id's, look at HTML::CalendarMonth::Locale->locales().
 
 =head1 METHODS
 
 All arguments appearing in [brackets] are optional, and do not represent
 anonymous array references.
 
-=over
+=head2 Constructor
 
-B<Constructor>
+=over
 
 =item new()
 
@@ -1105,10 +1078,12 @@ Specifies whether to display days of the week header. Default 1.
 
 =item locale
 
-Specifies a locale in which to render the calendar. Default is 'en_US'.
-See L<HTML::CalendarMonth::Locale> for more information. If for some
-reason you prefer to use different labels than those provided by
-C<locale>, see the C<alias> attribute below.
+Specifies the id of the locale in which to render the calendar. Default
+is 'en_US'. By default, this will also control determine which day is
+considered to be the first day of the week. See
+L<HTML::CalendarMonth::Locale> for more information. If for some reason
+you prefer to use different labels than those provided by C<locale>, see
+the C<alias> attribute below.
 
 =item full_days
 
@@ -1118,7 +1093,7 @@ the shortest (not guaranteed to be unique) abbreviations.
 
 =item full_months
 
-Specifies whether or not to use full month names or their abbriviated
+Specifies whether or not to use full month names or their abbreviated
 names. Default is 1, use full names. Use -1 for 'narrow' mode, the
 shortest (not guaranteed to be unique) abbreviations.
 
@@ -1136,8 +1111,8 @@ Specifies whether to display the week-of-year numbering. Default 0.
 =item week_begin
 
 Specify first day of the week, which can be 1..7, starting with Sunday.
-Defaults to 1, or Sunday. In order to specify Monday, set this to 2,
-and so on. By default, this is determined based on the locale.
+In order to specify Monday, set this to 2, and so on. By default, this
+is determined based on the locale.
 
 =item enable_css
 
@@ -1161,6 +1136,16 @@ these classes are:
   hcm-past     Set on the <lt>td<gt> tags for prior days, if present
   hcm-future   Set on the <lt>td<gt> tags for subsequent days, if present
 
+=item today
+
+Specify the value for 'today' if different from the local time as
+reported by the system clock (the default). If specified as two or less
+digits, it is assumed to be one of the days of the month in the current
+calendar. If more than two digits, it is assumed to be a epoch time in
+seconds. Otherwise it must be given as a string of the form 'YYYY-mm-
+dd'. Note that the default value as determined by the system clock uses
+localtime rather than gmtime.
+
 =item historic
 
 This option is ignored for dates that do not exceed the range of the built-
@@ -1177,7 +1162,7 @@ meaningless. Defaults to 1.
 
 =back
 
-B<Item Query Methods>
+=head2 Item Query Methods
 
 The following methods return lists of item *symbols* (28, 29, 'Thu',
 ...) that are related in some way to the provided list of items. The
@@ -1220,6 +1205,10 @@ Returns the column index for the first day of the month.
 
 Returns a list of all days of the month as numbers.
 
+=item week_nums()
+
+Returns a list of week-of-year numbers for this month.
+
 =item dayheaders()
 
 Returns a list of all day headers (Su..Sa)
@@ -1233,31 +1222,45 @@ Returns a list of all headers (month, year, dayheaders)
 Returns a list of all item symbols (day number, header values) in
 the calendar.
 
-=item first_col()
-
-Returns the index of the first column of the calendar. This is always 0.
-
 =item last_col()
 
 Returns the index of the last column of the calendar (note that this
 could be the week-of-year column if head_week is enabled).
 
-=item first_row()
+=item last_day_col()
 
-Returns the index of the first row of the calendar. This is always 0.
+Returns the index of the last column of the calendar containing days of
+the month (same as last_col() unless week-of-year is enabled).
 
 =item first_week_row()
 
 Returns the index of the first row of the calendar containing day items
-(ie, the first week). This could vary depending on header modes.
+(ie, the first week).
 
 =item last_row()
 
 Returns the index of the last row of the calendar.
 
+=item today()
+
+Returns the day of month for 'today', if present in the current
+calendar.
+
+=item past_days()
+
+Returns a list of days prior to 'today'. If 'today' is in a future
+month, all days are returned. If 'today' is in a past month, no days
+are returned.
+
+=item future_days()
+
+Returns a list of days after 'today'. If 'today' is in a past
+month, all days are returned. If 'today' is in a future month, no
+days are returned.
+
 =back
 
-B<Glob Methods>
+=head2 Glob Methods
 
 Glob methods return references that are functionally equivalent to an
 individual calendar cell. Mostly, they provide item based analogues to
@@ -1280,13 +1283,21 @@ Returns all cells containing the provided item symbols.
 
 Returns all cells in all rows occupied by the provided item symbols.
 
+=item item_day_row(item1, [item2, ...])
+
+Same as item_row() except excludes week-of-year cells, if present.
+
 =item item_col(item1, [item2, ...])
 
 Returns all cells in all columns occupied by the provided item symbols.
 
 =item item_daycol(item1, [item2, ...])
 
-Same as item_col(), except limits the cells to non header cells.
+Same as item_col() except limits the cells to non header cells.
+
+=item item_week_nums()
+
+Returns all week-of-year cells, if present.
 
 =item item_box(item1a, item1b, [item2a, item2b, ...])
 
@@ -1306,7 +1317,7 @@ Returns all cells in the calendar, including empty cells.
 
 =back
 
-B<Transformation Methods>
+=head2 Transformation Methods
 
 The following methods provide ways of translating between various item
 symbols, coordinates, and other representations.
@@ -1351,7 +1362,7 @@ must be present in the current calendar.
 
 =back
 
-B<Transformation Methods>
+=head2 Other Methods
 
 =over
 
@@ -1361,6 +1372,8 @@ Returns a simple style sheet as a string that can be used in an HTML
 document in conjunction with the classes assigned to elements when css
 is enabled.
 
+=back
+
 =head1 REQUIRES
 
 HTML::ElementTable
@@ -1391,4 +1404,4 @@ names, see http://www.loc.gov/standards/iso639-2/englangn.html
 
 HTML::ElementTable(3), HTML::Element(3), perl(1)
 
-=cut
+=for Pod::Coverage col_offset row_offset item_alias item_aliased last_week_row
